@@ -22,7 +22,6 @@ from bzrlib import (
     branch as _mod_branch,
     bzrdir,
     errors,
-    lazy_regex,
     merge,
     revision,
     revisionspec,
@@ -39,11 +38,6 @@ except ImportError:
     NoWhoami = object()
 
 
-try:
-    from debian import changelog
-except ImportError:
-    from debian_bundle import changelog
-
 try:
     MergeIntoMerger = merge.MergeIntoMerger
 except (AttributeError, NameError):
@@ -84,11 +78,18 @@ class SubstitutionVariable(object):
         """Replace name with value."""
         raise NotImplementedError(self.replace)
 
+    @classmethod
+    def available_in(cls, format):
+        """Check if this variable is available in a particular format."""
+        raise NotImplementedError(cls.available_in)
+
 
 class SimpleSubstitutionVariable(SubstitutionVariable):
 
     name = None
 
+    minimum_format = None
+
     def replace(self, value):
         if not self.name in value:
             return value
@@ -97,6 +98,10 @@ class SimpleSubstitutionVariable(SubstitutionVariable):
     def get(self):
         raise NotImplementedError(self.value)
 
+    @classmethod
+    def available_in(self, format):
+        return (format >= self.minimum_format)
+
 
 class BranchSubstitutionVariable(SimpleSubstitutionVariable):
 
@@ -122,6 +127,8 @@ class TimeVariable(SimpleSubstitutionVariable):
 
     name = "{time}"
 
+    minimum_format = 0.1
+
     def __init__(self, time):
         self._time = time
 
@@ -133,6 +140,8 @@ class DateVariable(SimpleSubstitutionVariable):
 
     name = "{date}"
 
+    minimum_format = 0.1
+
     def __init__(self, time):
         self._time = time
 
@@ -140,66 +149,10 @@ class DateVariable(SimpleSubstitutionVariable):
         return self._time.strftime("%Y%m%d")
 
 
-class DebUpstreamVariable(BranchSubstitutionVariable):
-
-    basename = "debupstream"
-
-    def __init__(self, branch_name, version):
-        super(DebUpstreamVariable, self).__init__(branch_name)
-        self._version = version
-
-    @classmethod
-    def from_changelog(cls, branch_name, changelog):
-        if len(changelog._blocks) > 0:
-            return cls(branch_name, changelog._blocks[0].version)
-        else:
-            return cls(branch_name, None)
-
-    def get(self):
-        if self._version is None:
-            raise SubstitutionUnavailable(self.name,
-                "No previous changelog to take the upstream version from")
-        # Should we include the epoch?
-        return self._version.upstream_version
-
-
-class DebVersionVariable(BranchSubstitutionVariable):
-
-    basename = "debversion"
-
-    def __init__(self, branch_name, version):
-        super(DebVersionVariable, self).__init__(branch_name)
-        self._version = version
-
-    @classmethod
-    def from_changelog(cls, branch_name, changelog):
-        if len(changelog._blocks) > 0:
-            return cls(branch_name, changelog._blocks[0].version)
-        else:
-            return cls(branch_name, None)
-
-    def get(self):
-        if self._version is None:
-            raise SubstitutionUnavailable(self.name,
-                "No previous changelog to take the version from")
-        return str(self._version)
-
-
-class DebUpstreamBaseVariable(DebUpstreamVariable):
-
-    basename = "debupstream-base"
-    version_regex = lazy_regex.lazy_compile(r'([~+])(svn[0-9]+|bzr[0-9]+|git[0-9a-f]+)')
-
-    def get(self):
-        version = super(DebUpstreamBaseVariable, self).get()
-        version = self.version_regex.sub("\\1", version)
-        if version[-1] not in ("~", "+"):
-            version += "+"
-        return version
-
-
 class RevisionVariable(BranchSubstitutionVariable):
 
+    minimum_format = 0.1
+
     def __init__(self, branch_name, branch, revid):
         super(RevisionVariable, self).__init__(branch_name)
         self.branch = branch
@@ -210,6 +163,8 @@ class RevnoVariable(RevisionVariable):
 
     basename = "revno"
 
+    minimum_format = 0.1
+
     def get_revno(self):
         try:
             revno = self.branch.revision_id_to_revno(self.revid)
@@ -235,6 +190,8 @@ class RevtimeVariable(RevisionVariable):
 
     basename = "revtime"
 
+    minimum_format = 0.4
+
     def get(self):
         rev = self.branch.repository.get_revision(self.revid)
         return time.strftime("%Y%m%d%H%M", time.gmtime(rev.timestamp))
@@ -244,6 +201,8 @@ class RevdateVariable(RevisionVariable):
 
     basename = "revdate"
 
+    minimum_format = 0.4
+
     def get(self):
         rev = self.branch.repository.get_revision(self.revid)
         return time.strftime("%Y%m%d", time.gmtime(rev.timestamp))
@@ -271,6 +230,8 @@ class SubversionRevnumVariable(RevisionVariable):
 
     basename = "svn-revno"
 
+    minimum_format = 0.4
+
     def get(self):
         rev = self.branch.repository.get_revision(self.revid)
         try:
@@ -304,6 +265,8 @@ class GitCommitVariable(RevisionVariable):
 
     basename = "git-commit"
 
+    minimum_format = 0.4
+
     def get(self):
         rev = self.branch.repository.get_revision(self.revid)
         try:
@@ -319,6 +282,8 @@ class LatestTagVariable(RevisionVariable):
 
     basename = "latest-tag"
 
+    minimum_format = 0.4
+
     def get(self):
         reverse_tag_dict = self.branch.tags.get_reverse_tag_dict()
         self.branch.lock_read()
@@ -334,39 +299,10 @@ class LatestTagVariable(RevisionVariable):
             self.branch.unlock()
 
 
-ok_to_preserve = [DebUpstreamVariable, DebUpstreamBaseVariable,
-    DebVersionVariable]
-# The variables that don't require substitution in their name
-simple_vars = [TimeVariable, DateVariable]
 branch_vars = [RevnoVariable, SubversionRevnumVariable,
-    GitCommitVariable, LatestTagVariable, DebVersionVariable,
-    DebUpstreamBaseVariable, DebUpstreamVariable, RevdateVariable,
+    GitCommitVariable, LatestTagVariable, RevdateVariable,
     RevtimeVariable]
-
-
-def check_expanded_deb_version(base_branch):
-    checked_version = base_branch.deb_version
-    if checked_version is None:
-        return
-    for token in ok_to_preserve:
-        if issubclass(token, BranchSubstitutionVariable):
-            for name in base_branch.list_branch_names():
-                checked_version = checked_version.replace(
-                    token.determine_name(name), "")
-            checked_version = checked_version.replace(
-                    token.determine_name(None), "")
-        else:
-            checked_version = checked_version.replace(
-                token.name, "")
-    if "{" in checked_version:
-        available_tokens = [var.name for var in simple_vars]
-        for var_kls in branch_vars:
-            for name in base_branch.list_branch_names():
-                available_tokens.append(var_kls.determine_name(name))
-            available_tokens.append(var_kls.determine_name(None))
-        raise errors.BzrCommandError("deb-version not fully "
-                "expanded: %s. Valid substitutions are: %s"
-                % (base_branch.deb_version, available_tokens))
+simple_vars = [TimeVariable, DateVariable]
 
 
 class CommandFailedError(errors.BzrError):
@@ -558,6 +494,14 @@ def nest_part_branch(child_branch, tree_to, br_to, subpath, target_subdir=None):
         try:
             if target_subdir is None:
                 target_subdir = os.path.basename(subpath)
+            # Create any missing parent directories
+            target_subdir_parent = os.path.dirname(target_subdir)
+            missing = []
+            while tree_to.path2id(target_subdir_parent) is None:
+                missing.append(target_subdir_parent)
+                target_subdir_parent = os.path.dirname(target_subdir_parent)
+            for path in reversed(missing):
+                tree_to.mkdir(path)
             merger = MergeIntoMerger(this_tree=tree_to, other_tree=other_tree,
                 other_branch=child_branch.branch, target_subdir=target_subdir,
                 source_subpath=subpath, other_rev_id=child_branch.revid)
@@ -599,7 +543,8 @@ def _resolve_revisions_recurse(new_branch, substitute_branch_vars,
     new_branch.branch.lock_read()
     try:
         new_branch.resolve_revision_id()
-        substitute_branch_vars(new_branch.name, new_branch.branch, new_branch.revid)
+        if substitute_branch_vars is not None:
+            substitute_branch_vars(new_branch.name, new_branch.branch, new_branch.revid)
         if (if_changed_from is not None
                 and (new_branch.revspec is not None
                         or if_changed_from.revspec is not None)):
@@ -628,10 +573,11 @@ def _resolve_revisions_recurse(new_branch, substitute_branch_vars,
         new_branch.branch.unlock()
 
 
-def resolve_revisions(base_branch, if_changed_from=None):
+def resolve_revisions(base_branch, if_changed_from=None, substitute_branch_vars=None):
     """Resolve all the unknowns in base_branch.
 
-    This walks the RecipeBranch and substitutes in revnos and deb_version.
+    This walks the RecipeBranch and calls substitute_branch_vars for
+    each child branch.
 
     If if_changed_from is not None then it should be a second RecipeBranch
     to compare base_branch against. If the shape, or the revision ids differ
@@ -639,6 +585,8 @@ def resolve_revisions(base_branch, if_changed_from=None):
 
     :param base_branch: the RecipeBranch we plan to build.
     :param if_changed_from: the RecipeBranch that we want to compare against.
+    :param substitute_branch_vars: Callable called for
+        each branch with (name, bzr branch and last revision)
     :return: False if if_changed_from is not None, and the shape and revisions
         of the two branches don't differ. True otherwise.
     """
@@ -648,12 +596,16 @@ def resolve_revisions(base_branch, if_changed_from=None):
     if_changed_from_revisions = if_changed_from
     if changed:
         if_changed_from_revisions = None
+
+    if substitute_branch_vars is not None:
+        real_subsitute_branch_vars = lambda n, b, r: substitute_branch_vars(base_branch, n, b, r)
+    else:
+        real_subsitute_branch_vars = None
     changed_revisions = _resolve_revisions_recurse(base_branch,
-            base_branch.substitute_branch_vars,
+            real_subsitute_branch_vars,
             if_changed_from=if_changed_from_revisions)
     if not changed:
         changed = changed_revisions
-    check_expanded_deb_version(base_branch)
     if if_changed_from is not None and not changed:
         return False
     return True
@@ -984,66 +936,6 @@ class BaseRecipeBranch(RecipeBranch):
         self.deb_version = deb_version
         self.format = format
 
-    def substitute_branch_vars(self, branch_name, branch, revid):
-        """Substitute the branch variables for the given branch name in deb_version.
-
-        Where deb_version has a place to substitute the revno for a branch
-        this will substitute it for the given branch name.
-
-        :param branch_name: the name of the RecipeBranch to substitute.
-        :param branch: Branch object for the branch
-        :param revid: Revision id in the branch for which to return the revno
-        """
-        if self.deb_version is None:
-            return
-        revno_var = RevnoVariable(branch_name, branch, revid)
-        self.deb_version = revno_var.replace(self.deb_version)
-        if self.format in (0.1, 0.2, 0.3):
-            return
-        svn_revno_var = SubversionRevnumVariable(branch_name, branch, revid)
-        self.deb_version = svn_revno_var.replace(self.deb_version)
-        git_commit_var = GitCommitVariable(branch_name, branch, revid)
-        self.deb_version = git_commit_var.replace(self.deb_version)
-        latest_tag_var = LatestTagVariable(branch_name, branch, revid)
-        self.deb_version = latest_tag_var.replace(self.deb_version)
-        revdate_var = RevdateVariable(branch_name, branch, revid)
-        self.deb_version = revdate_var.replace(self.deb_version)
-        revtime_var = RevtimeVariable(branch_name, branch, revid)
-        self.deb_version = revtime_var.replace(self.deb_version)
-        tree = branch.repository.revision_tree(revid)
-        cl_file_id = tree.path2id("debian/changelog")
-        if cl_file_id is not None:
-            tree.lock_read()
-            try:
-                cl = changelog.Changelog(tree.get_file(cl_file_id))
-                self.substitute_changelog_vars(branch_name, cl)
-            finally:
-                tree.unlock()
-
-    def substitute_changelog_vars(self, branch_name, changelog):
-        """Substitute variables related from a changelog.
-
-        :param branch_name: Branch name (None for root branch)
-        :param changelog: Changelog object to use
-        """
-        debupstream_var = DebUpstreamVariable.from_changelog(branch_name, changelog)
-        self.deb_version = debupstream_var.replace(self.deb_version)
-        debupstreambase_var = DebUpstreamBaseVariable.from_changelog(
-            branch_name, changelog)
-        self.deb_version = debupstreambase_var.replace(self.deb_version)
-        pkgversion_var = DebVersionVariable.from_changelog(branch_name, changelog)
-        self.deb_version = pkgversion_var.replace(self.deb_version)
-
-    def substitute_time(self, time):
-        """Substitute the time in to deb_version if needed.
-
-        :param time: a datetime.datetime with the desired time.
-        """
-        if self.deb_version is None:
-            return
-        self.deb_version = TimeVariable(time).replace(self.deb_version)
-        self.deb_version = DateVariable(time).replace(self.deb_version)
-
     def _add_child_branches_to_manifest(self, child_branches, indent_level):
         manifest = ""
         for instruction in child_branches:
