@@ -26,13 +26,13 @@ from bzrlib import (
     )
 from bzrlib.branch import Branch
 from bzrlib.tests import (
-        TestCaseWithTransport,
-        TestSkipped,
-        )
-from bzrlib.tests.features import Feature
+    TestCaseWithTransport,
+    )
 
-
-from bzrlib.plugins.builder.tests import PristineTarFeature
+from bzrlib.plugins.builder.tests import (
+    Feature,
+    PristineTarFeature,
+    )
 
 try:
     from debian import changelog
@@ -359,7 +359,7 @@ class BlackboxBuilderTests(TestCaseWithTransport):
         new_cl_contents = ("package (1) unstable; urgency=low\n\n"
                 "  * Auto build.\n\n -- M. Maintainer <maint@maint.org>  ")
         actual_cl_contents = self._get_file_contents(
-            "working/package-1/debian/changelog")
+            "working/test-1/debian/changelog")
         self.assertStartsWith(actual_cl_contents, new_cl_contents)
         for fn in os.listdir("working"):
             self.assertFalse(fn.endswith(".changes"))
@@ -381,7 +381,7 @@ class BlackboxBuilderTests(TestCaseWithTransport):
     def test_cmd_dailydeb_with_version_from_changelog(self):
         self.requireFeature(NotUnderFakeRootFeature)
         self.make_simple_package("source")
-        self.build_tree_contents([("test.recipe", "# bzr-builder format 0.1 "
+        self.build_tree_contents([("test.recipe", "# bzr-builder format 0.4 "
                     "deb-version {debversion}-2\nsource 1\n")])
         out, err = self.run_bzr(
             "dailydeb --allow-fallback-to-native -q test.recipe working")
