@@ -13,9 +13,7 @@
 # You should have received a copy of the GNU General Public License along 
 # with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import datetime
 import os
-import textwrap
 
 from bzrlib import (
         errors,
@@ -23,41 +21,27 @@ from bzrlib import (
         workingtree,
         )
 from bzrlib.tests import (
-        TestCase,
         TestCaseInTempDir,
         TestCaseWithTransport,
         )
 from bzrlib.plugins.builder.recipe import (
         BaseRecipeBranch,
         build_tree,
-        DebUpstreamBaseVariable,
-        DebUpstreamVariable,
         ensure_basedir,
         InstructionParseError,
         ForbiddenInstructionError,
         MERGE_INSTRUCTION,
         NEST_INSTRUCTION,
         NEST_PART_INSTRUCTION,
-        DebVersionVariable,
         pull_or_branch,
         RecipeParser,
         RecipeBranch,
         RecipeParseError,
-        resolve_revisions,
         RUN_INSTRUCTION,
         SAFE_INSTRUCTIONS,
-        SubstitutionUnavailable,
         USAGE,
         )
 
-try:
-    from debian import changelog
-except ImportError:
-    # In older versions of python-debian the main package was named 
-    # debian_bundle
-    from debian_bundle import changelog
-
-
 class RecipeParserTests(TestCaseInTempDir):
 
     deb_version = "0.1-{revno}"
@@ -654,6 +638,33 @@ class BuildTreeTests(TestCaseWithTransport):
         self.assertEqual(source1_rev_id, base_branch.revid)
         self.assertEqual(source2_rev_id, merged_branch.revid)
 
+    def test_build_tree_implicit_dir(self):
+        # Branches nested into non-existant directories trigger creation of
+        # those directories.
+        source1 = self.make_source_branch("source1")
+        source2 = self.make_source_branch("source2")
+        source3 = self.make_source_branch("source3")
+        self.build_tree_contents([
+            ("source2/file", "new file"),
+            ("source3/yetanotherfile", "rugby")])
+        source2.add(["file"])
+        source2.commit("two")
+        source3.add(["yetanotherfile"])
+        source3.commit("three")
+        base_branch = BaseRecipeBranch("source1", "1", 0.4)
+        merged_branch_2 = RecipeBranch("merged", "source2")
+        # Merge source2 into path implicit/b
+        base_branch.nest_part_branch(merged_branch_2, ".",
+            target_subdir="implicit/b")
+        # Merge source3 into path implicit/moreimplicit/c
+        merged_branch_3 = RecipeBranch("merged", "source3")
+        base_branch.nest_part_branch(merged_branch_3, ".",
+            target_subdir="moreimplicit/another/c")
+        build_tree(base_branch, "target")
+        self.check_file_contents("target/implicit/b/file", "new file")
+        self.check_file_contents("target/moreimplicit/another/c/yetanotherfile",
+            "rugby")
+
     def test_build_tree_nest_part(self):
         """A recipe can specify a merge of just part of an unrelated tree."""
         source1 = self.make_source_branch("source1")
@@ -723,7 +734,7 @@ class BuildTreeTests(TestCaseWithTransport):
         self.assertPathExists("target")
         tree = workingtree.WorkingTree.open("target")
         last_revid = tree.last_revision()
-        previous_revid = tree.branch.revision_history()[-2]
+        previous_revid = tree.branch.repository.get_revision(tree.branch.last_revision()).parent_ids[0]
         last_revtree = tree.branch.repository.revision_tree(last_revid)
         previous_revtree = tree.branch.repository.revision_tree(previous_revid)
         self.assertEqual([previous_revid, source3_rev_id],
@@ -932,214 +943,6 @@ class BuildTreeTests(TestCaseWithTransport):
                     "revspec at the end of the merge line?"))
 
 
-class ResolveRevisionsTests(TestCaseWithTransport):
-
-    def test_unchanged(self):
-        source =self.make_branch_and_tree("source")
-        revid = source.commit("one")
-        branch1 = BaseRecipeBranch("source", "{revno}", 0.2, revspec="1")
-        branch2 = BaseRecipeBranch("source", "{revno}", 0.2,
-                revspec="revid:%s" % revid)
-        self.assertEqual(False, resolve_revisions(branch1,
-                    if_changed_from=branch2))
-        self.assertEqual("source", branch1.url)
-        self.assertEqual(revid, branch1.revid)
-        self.assertEqual("1", branch1.revspec)
-        self.assertEqual("1", branch1.deb_version)
-
-    def test_unchanged_not_explicit(self):
-        source =self.make_branch_and_tree("source")
-        revid = source.commit("one")
-        branch1 = BaseRecipeBranch("source", "{revno}", 0.2)
-        branch2 = BaseRecipeBranch("source", "{revno}", 0.2,
-                revspec="revid:%s" % revid)
-        self.assertEqual(False, resolve_revisions(branch1,
-                    if_changed_from=branch2))
-        self.assertEqual("source", branch1.url)
-        self.assertEqual(revid, branch1.revid)
-        self.assertEqual(None, branch1.revspec)
-        self.assertEqual("1", branch1.deb_version)
-
-    def test_unchanged_multilevel(self):
-        source =self.make_branch_and_tree("source")
-        revid = source.commit("one")
-        branch1 = BaseRecipeBranch("source", "{revno}", 0.2)
-        branch2 = RecipeBranch("nested1", "source")
-        branch3 = RecipeBranch("nested2", "source")
-        branch2.nest_branch("bar", branch3)
-        branch1.nest_branch("foo", branch2)
-        branch4 = BaseRecipeBranch("source", "{revno}", 0.2,
-                revspec="revid:%s" % revid)
-        branch5 = RecipeBranch("nested1", "source",
-                revspec="revid:%s" % revid)
-        branch6 = RecipeBranch("nested2", "source",
-                revspec="revid:%s" % revid)
-        branch5.nest_branch("bar", branch6)
-        branch4.nest_branch("foo", branch5)
-        self.assertEqual(False, resolve_revisions(branch1,
-                    if_changed_from=branch4))
-        self.assertEqual("source", branch1.url)
-        self.assertEqual(revid, branch1.revid)
-        self.assertEqual(None, branch1.revspec)
-        self.assertEqual("1", branch1.deb_version)
-
-    def test_changed(self):
-        source =self.make_branch_and_tree("source")
-        revid = source.commit("one")
-        branch1 = BaseRecipeBranch("source", "{revno}", 0.2, revspec="1")
-        branch2 = BaseRecipeBranch("source", "{revno}", 0.2,
-                revspec="revid:foo")
-        self.assertEqual(True, resolve_revisions(branch1,
-                    if_changed_from=branch2))
-        self.assertEqual("source", branch1.url)
-        self.assertEqual(revid, branch1.revid)
-        self.assertEqual("1", branch1.revspec)
-        self.assertEqual("1", branch1.deb_version)
-
-    def test_changed_shape(self):
-        source =self.make_branch_and_tree("source")
-        revid = source.commit("one")
-        branch1 = BaseRecipeBranch("source", "{revno}", 0.2, revspec="1")
-        branch2 = BaseRecipeBranch("source", "{revno}", 0.2,
-                revspec="revid:%s" % revid)
-        branch3 = RecipeBranch("nested", "source")
-        branch1.nest_branch("foo", branch3)
-        self.assertEqual(True, resolve_revisions(branch1,
-                    if_changed_from=branch2))
-        self.assertEqual("source", branch1.url)
-        self.assertEqual(revid, branch1.revid)
-        self.assertEqual("1", branch1.revspec)
-        self.assertEqual("1", branch1.deb_version)
-
-    def test_changed_command(self):
-        source =self.make_branch_and_tree("source")
-        source.commit("one")
-        branch1 = BaseRecipeBranch("source", "{revno}", 0.2)
-        branch2 = BaseRecipeBranch("source", "{revno}", 0.2)
-        branch1.run_command("touch test1")
-        branch2.run_command("touch test2")
-        self.assertEqual(True, resolve_revisions(branch1,
-                    if_changed_from=branch2))
-        self.assertEqual("source", branch1.url)
-
-    def test_unchanged_command(self):
-        source =self.make_branch_and_tree("source")
-        source.commit("one")
-        branch1 = BaseRecipeBranch("source", "{revno}", 0.2)
-        branch2 = BaseRecipeBranch("source", "{revno}", 0.2)
-        branch1.run_command("touch test1")
-        branch2.run_command("touch test1")
-        self.assertEqual(False, resolve_revisions(branch1,
-                    if_changed_from=branch2))
-        self.assertEqual("source", branch1.url)
-
-    def test_substitute(self):
-        source =self.make_branch_and_tree("source")
-        revid1 = source.commit("one")
-        source.commit("two")
-        branch1 = BaseRecipeBranch("source",
-                "{revno}-{revno:packaging}", 0.2, revspec="1")
-        branch2 = RecipeBranch("packaging", "source")
-        branch1.nest_branch("debian", branch2)
-        self.assertEqual(True, resolve_revisions(branch1))
-        self.assertEqual("source", branch1.url)
-        self.assertEqual(revid1, branch1.revid)
-        self.assertEqual("1", branch1.revspec)
-        self.assertEqual("1-2", branch1.deb_version)
-
-    def test_substitute_supports_debupstream(self):
-        # resolve_revisions should leave debupstream parameters alone and not
-        # complain.
-        source =self.make_branch_and_tree("source")
-        source.commit("one")
-        source.commit("two")
-        branch1 = BaseRecipeBranch("source", "{debupstream}-{revno}", 0.2)
-        resolve_revisions(branch1)
-        self.assertEqual("{debupstream}-2", branch1.deb_version)
-
-    def test_subsitute_not_fully_expanded(self):
-        source =self.make_branch_and_tree("source")
-        source.commit("one")
-        source.commit("two")
-        branch1 = BaseRecipeBranch("source", "{revno:packaging}", 0.2)
-        self.assertRaises(errors.BzrCommandError, resolve_revisions, branch1)
-
-    def test_substitute_svn_not_svn(self):
-        br = self.make_branch("source")
-        source = br.create_checkout("checkout")
-        source.commit("one")
-        source.commit("two")
-        branch1 = BaseRecipeBranch("source", "foo-{svn-revno}", 0.4)
-        e = self.assertRaises(errors.BzrCommandError, resolve_revisions,
-            branch1)
-        self.assertTrue(str(e).startswith("unable to expand {svn-revno} "),
-            e)
-
-    def test_substitute_svn(self):
-        br = self.make_branch("source")
-        source = br.create_checkout("checkout")
-        source.commit("one")
-        source.commit("two",
-            rev_id="svn-v4:be7e6eca-30d4-0310-a8e5-ac0d63af7070:trunk:5344")
-        branch1 = BaseRecipeBranch("source", "foo-{svn-revno}", 0.4)
-        resolve_revisions(branch1)
-        self.assertEqual("foo-5344", branch1.deb_version)
-
-    def test_substitute_git_not_git(self):
-        source = self.make_branch_and_tree("source")
-        source.commit("one")
-        source.commit("two")
-        branch1 = BaseRecipeBranch("source", "foo-{git-commit}", 0.4)
-        e = self.assertRaises(errors.BzrCommandError, resolve_revisions,
-            branch1)
-        self.assertTrue(str(e).startswith("unable to expand {git-commit} "),
-            e)
-
-    def test_substitute_git(self):
-        source = self.make_branch_and_tree("source")
-        source.commit("one", 
-            rev_id="git-v1:a029d7b2cc83c26a53d8b2a24fa12c340fcfac58")
-        branch1 = BaseRecipeBranch("source", "foo-{git-commit}", 0.4)
-        resolve_revisions(branch1)
-        self.assertEqual("foo-a029d7b", branch1.deb_version)
-
-    def test_latest_tag(self):
-        source = self.make_branch_and_tree("source")
-        revid = source.commit("one")
-        source.branch.tags.set_tag("millbank", revid)
-        source.commit("two")
-        branch1 = BaseRecipeBranch("source", "foo-{latest-tag}", 0.4)
-        resolve_revisions(branch1)
-        self.assertEqual("foo-millbank", branch1.deb_version)
-
-    def test_latest_tag_no_tag(self):
-        source = self.make_branch_and_tree("source")
-        revid = source.commit("one")
-        source.commit("two")
-        branch1 = BaseRecipeBranch("source", "foo-{latest-tag}", 0.4)
-        e = self.assertRaises(errors.BzrCommandError, resolve_revisions, branch1)
-        self.assertTrue(str(e).startswith("No tags set on branch None mainline"),
-            e)
-
-    def test_substitute_revdate(self):
-        br = self.make_branch("source")
-        source = br.create_checkout("checkout")
-        source.commit("one")
-        source.commit("two", timestamp=1307708628, timezone=0)
-        branch1 = BaseRecipeBranch("source", "foo-{revdate}", 0.4)
-        resolve_revisions(branch1)
-        self.assertEqual("foo-20110610", branch1.deb_version)
-
-    def test_substitute_revtime(self):
-        br = self.make_branch("source")
-        source = br.create_checkout("checkout")
-        source.commit("one")
-        source.commit("two", timestamp=1307708628, timezone=0)
-        branch1 = BaseRecipeBranch("source", "foo-{revtime}", 0.4)
-        resolve_revisions(branch1)
-        self.assertEqual("foo-201106101223", branch1.deb_version)
-
-
 class StringifyTests(TestCaseInTempDir):
 
     def test_missing_debversion(self):
@@ -1317,77 +1120,6 @@ class RecipeBranchTests(TestCaseWithTransport):
         rbranch2 = RecipeBranch("name", "other_url2")
         self.assertTrue(rbranch1.different_shape_to(rbranch2))
 
-    def test_substitute_time(self):
-        time = datetime.datetime.utcfromtimestamp(1)
-        base_branch = BaseRecipeBranch("base_url", "1-{time}", 0.2)
-        base_branch.substitute_time(time)
-        self.assertEqual("1-197001010000", base_branch.deb_version)
-        base_branch.substitute_time(time)
-        self.assertEqual("1-197001010000", base_branch.deb_version)
-
-    def test_substitute_date(self):
-        time = datetime.datetime.utcfromtimestamp(1)
-        base_branch = BaseRecipeBranch("base_url", "1-{date}", 0.2)
-        base_branch.substitute_time(time)
-        self.assertEqual("1-19700101", base_branch.deb_version)
-        base_branch.substitute_time(time)
-        self.assertEqual("1-19700101", base_branch.deb_version)
-
-    def test_substitute_branch_vars(self):
-        base_branch = BaseRecipeBranch("base_url", "1", 0.2)
-        wt = self.make_branch_and_tree("br")
-        revid = wt.commit("acommit")
-        base_branch.substitute_branch_vars(None, wt.branch, revid)
-        self.assertEqual("1", base_branch.deb_version)
-        base_branch.substitute_branch_vars(None, wt.branch, revid)
-        self.assertEqual("1", base_branch.deb_version)
-        base_branch = BaseRecipeBranch("base_url", "{revno}", 0.2)
-        base_branch.substitute_branch_vars(None, wt.branch, revid)
-        self.assertEqual("1", base_branch.deb_version)
-        base_branch = BaseRecipeBranch("base_url", "{revno}", 0.2)
-        base_branch.substitute_branch_vars("foo", wt.branch, revid)
-        self.assertEqual("{revno}", base_branch.deb_version)
-        base_branch.substitute_branch_vars("foo", wt.branch, revid)
-        self.assertEqual("{revno}", base_branch.deb_version)
-        base_branch = BaseRecipeBranch("base_url", "{revno:foo}", 0.2)
-        base_branch.substitute_branch_vars("foo", wt.branch, revid)
-        self.assertEqual("1", base_branch.deb_version)
-
-    def test_substitute_branch_vars_debupstream(self):
-        wt = self.make_branch_and_tree("br")
-        revid1 = wt.commit("acommit")
-        cl_contents = ("package (0.1-1) unstable; urgency=low\n  * foo\n"
-                    " -- maint <maint@maint.org>  Tue, 04 Aug 2009 "
-                    "10:03:10 +0100\n")
-        self.build_tree_contents(
-            [("br/debian/", ), ('br/debian/changelog', cl_contents)])
-        wt.add(['debian', 'debian/changelog'])
-        revid2 = wt.commit("with changelog")
-        base_branch = BaseRecipeBranch("base_url", "{debupstream}", 0.4)
-        # No changelog file, so no substitution
-        base_branch.substitute_branch_vars(None, wt.branch, revid1)
-        self.assertEqual("{debupstream}", base_branch.deb_version)
-        base_branch.substitute_branch_vars(None, wt.branch, revid2)
-        self.assertEqual("0.1", base_branch.deb_version)
-        base_branch = BaseRecipeBranch("base_url", "{debupstream:tehname}", 0.4)
-        base_branch.substitute_branch_vars("tehname", wt.branch, revid2)
-        self.assertEqual("0.1", base_branch.deb_version)
-
-    def test_substitute_branch_vars_debupstream_pre_0_4(self):
-        wt = self.make_branch_and_tree("br")
-        cl_contents = ("package (0.1-1) unstable; urgency=low\n  * foo\n"
-                    " -- maint <maint@maint.org>  Tue, 04 Aug 2009 "
-                    "10:03:10 +0100\n")
-        self.build_tree_contents(
-            [("br/debian/", ), ('br/debian/changelog', cl_contents)])
-        wt.add(['debian', 'debian/changelog'])
-        revid = wt.commit("with changelog")
-        # In recipe format < 0.4 {debupstream} gets replaced from the resulting
-        # tree, not from the branch vars.
-        base_branch = BaseRecipeBranch("base_url", "{debupstream}", 0.2)
-        base_branch.substitute_branch_vars(None, wt.branch, revid)
-        self.assertEqual("{debupstream}", base_branch.deb_version)
-
     def test_list_branch_names(self):
         base_branch = BaseRecipeBranch("base_url", "1", 0.2)
         base_branch.merge_branch(RecipeBranch("merged", "merged_url"))
@@ -1431,73 +1163,3 @@ class RecipeBranchTests(TestCaseWithTransport):
         self.assertEqual([
             cmd, nest, merge_into_nested],
             list(base_branch.iter_all_instructions()))
-
-
-class DebUpstreamVariableTests(TestCase):
-
-    def write_changelog(self, version):
-        contents = textwrap.dedent("""
-            package (%s) experimental; urgency=low
-
-              * Initial release. (Closes: #XXXXXX)
-
-             -- Jelmer Vernooij <jelmer@debian.org>  Thu, 19 May 2011 10:07:41 +0100
-            """ % version)[1:]
-        return changelog.Changelog(file=contents)
-
-    def test_empty_changelog(self):
-        var = DebUpstreamVariable.from_changelog(None, changelog.Changelog())
-        self.assertRaises(SubstitutionUnavailable, var.get)
-
-    def test_version(self):
-        var = DebUpstreamVariable.from_changelog(None,
-            self.write_changelog("2.3"))
-        self.assertEquals("2.3", var.get())
-
-    def test_epoch(self):
-        # The epoch is (currently) ignored by {debupstream}.
-        var = DebUpstreamVariable.from_changelog(None,
-            self.write_changelog("2:2.3"))
-        self.assertEquals("2.3", var.get())
-
-    def test_base_without_snapshot(self):
-        var = DebUpstreamBaseVariable.from_changelog(None,
-            self.write_changelog("2.4"))
-        self.assertEquals("2.4+", var.get())
-
-    def test_base_with_svn_snapshot(self):
-        var = DebUpstreamBaseVariable.from_changelog(None,
-            self.write_changelog("2.4~svn4"))
-        self.assertEquals("2.4~", var.get())
-
-    def test_base_with_bzr_snapshot(self):
-        var = DebUpstreamBaseVariable.from_changelog(None,
-            self.write_changelog("2.4+bzr343"))
-        self.assertEquals("2.4+", var.get())
-
-
-class DebVersionVariableTests(TestCase):
-
-    def write_changelog(self, version):
-        contents = textwrap.dedent("""
-            package (%s) experimental; urgency=low
-
-              * Initial release. (Closes: #XXXXXX)
-
-             -- Jelmer Vernooij <jelmer@debian.org>  Thu, 19 May 2011 10:07:41 +0100
-            """ % version)[1:]
-        return changelog.Changelog(file=contents)
-
-    def test_empty_changelog(self):
-        var = DebVersionVariable.from_changelog(None, changelog.Changelog())
-        self.assertRaises(SubstitutionUnavailable, var.get)
-
-    def test_simple(self):
-        var = DebVersionVariable.from_changelog(
-            None, self.write_changelog("2.3-1"))
-        self.assertEquals("2.3-1", var.get())
-
-    def test_epoch(self):
-        var = DebVersionVariable.from_changelog(
-            None, self.write_changelog("4:2.3-1"))
-        self.assertEquals("4:2.3-1", var.get())
